"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const ParserError_1 = require("../errors/ParserError");
const ParserErrors_1 = require("../errors/ParserErrors");
class ParamParser {
    constructor() {
        this.params = [];
        this.onError = null;
    }
    withCustomErrorHandler(onError) {
        this.onError = onError;
        return this;
    }
    getAs(path, as, options) {
        return this.get(path, Object.assign({}, options, { as }));
    }
    get(path, options) {
        return this.add(Object.assign({}, options, { path }));
    }
    addAll(options) {
        Object.keys(options).forEach(key => this.add(Object.assign({}, options[key], { path: key })));
        return this;
    }
    add(options) {
        this.params.push(options);
        return this;
    }
    parse(data) {
        return (this.params || []).reduce((acc, options) => {
            const { path, req, def, as, validate, sanitize, onError, } = options;
            const errorHandler = (value, errorCode) => {
                const errorId = ParserError_1.ParserErrorIds[errorCode];
                const parserError = ParserErrors_1.ParserErrorBuilders[errorId](path, value);
                _.isFunction(onError) && onError(path, value, parserError);
                _.isFunction(this.onError) && this.onError(path, value, parserError);
            };
            const defaultValue = _.isFunction(def)
                ? def()
                : def;
            const value = _.get(data, path) || defaultValue;
            const required = req == null || req;
            if (value == null && required) {
                errorHandler(value, ParserError_1.ParserErrorCodes.RequiredPropNotSet);
                return acc;
            }
            const sanitizedValue = sanitize != null
                ? _.castArray(sanitize).reduce((v, f) => f(v + ''), value)
                : value;
            const allValid = validate != null
                ? _.castArray(validate).reduce((v, f) => v && f(sanitizedValue), true)
                : true;
            if (!allValid) {
                errorHandler(sanitizedValue, ParserError_1.ParserErrorCodes.PropIsInvalid);
                return acc;
            }
            const key = _.isString(as)
                ? as
                : _.last(path.split('.'));
            return _.set(acc, key, sanitizedValue);
        }, {});
    }
}
exports.ParamParser = ParamParser;
//# sourceMappingURL=ParamParser.js.map